@@ -31,8 +31,11 @@ class NodeState:
         if node in self.nodes:
             self.nodes[node] = time.time()
 
+    def get_all_nodes(self):
+        return tuple(self.nodes.keys())
+
     def get_online_nodes(self):
-        return (x for x, y in self.nodes.items() if y is None or time.time() > y + self.RETRY_INTERVAL)
+        return tuple(x for x, y in self.nodes.items() if y is None or time.time() > y + self.RETRY_INTERVAL)
 
 
 NODE_STATE = NodeState(multiprocessing.Manager().dict())
@@ -157,6 +160,7 @@ class CursorWrapper:
         if self._backend.failover_active:
             error_code = exc.args[0]
             if error_code in (
+                    1047,  # Unknown command (wsrep_reject_queries)
                     2006,  # MySQL server has gone away
                     2013,  # Lost connection to MySQL server during query
             ):
@@ -223,9 +227,9 @@ class DatabaseWrapper(base.DatabaseWrapper):
 
     def connect_to_node(self, primary=True):
         if primary:
-            nodes = sorted(list(NODE_STATE.get_online_nodes()))
+            nodes = sorted(NODE_STATE.get_online_nodes() or NODE_STATE.get_all_nodes())
         else:
-            nodes = list(NODE_STATE.get_online_nodes())
+            nodes = list(NODE_STATE.get_online_nodes() or NODE_STATE.get_all_nodes())
             random.shuffle(nodes)
             preferred_host = self.base_settings.get('HOST', '')
             if preferred_host:
@@ -248,18 +252,36 @@ class DatabaseWrapper(base.DatabaseWrapper):
                     self._secondary_wrapper = base.DatabaseWrapper(settings_dict, alias=self.alias)
                     self._secondary_wrapper.connect()
                     connection = self._secondary_wrapper.connection
-                cursor = connection.cursor()
-                cursor.execute(
-                    "SELECT variable_value "
-                    "FROM information_schema.global_status "
-                    "WHERE variable_name = 'wsrep_ready'")
-                result = cursor.fetchone()
-                cursor.close()
-                if result[0] == 'ON':
-                    NODE_STATE.mark_online(node)
-                    break
-                else:
-                    raise base.Database.Error('wsrep not ready')
+                with connection.cursor() as cursor:
+                    cursor.execute(
+                        "SELECT variable_name, variable_value "
+                        "FROM information_schema.global_status "
+                        "WHERE variable_name IN ("
+                        "'WSREP_CLUSTER_STATUS', 'WSREP_LOCAL_STATE', 'WSREP_READY'"
+                        ") "
+                        "UNION "
+                        "SELECT variable_name, variable_value "
+                        "FROM information_schema.global_variables "
+                        "WHERE variable_name IN ("
+                        "'WSREP_DESYNC', 'WSREP_REJECT_QUERIES', 'WSREP_SST_DONOR_REJECTS_QUERIES'"
+                        ")"
+                    )
+                    results = {k.upper(): v.upper() for k, v in cursor.fetchall()}
+                if results['WSREP_READY'] != 'ON':
+                    raise base.Database.Error('WSREP_READY: %s' % results['WSREP_READY'])
+                if results['WSREP_CLUSTER_STATUS'] != 'PRIMARY':
+                    raise base.Database.Error('WSREP_CLUSTER_STATUS: %s' % results['WSREP_CLUSTER_STATUS'])
+                if results['WSREP_DESYNC'] != 'OFF':
+                    raise base.Database.Error('WSREP_DESYNC')
+                if results['WSREP_LOCAL_STATE'] != '4':
+                    if results['WSREP_LOCAL_STATE'] != '2':
+                        raise base.Database.Error('WSREP_LOCAL_STATE: %s' % results['WSREP_LOCAL_STATE'])
+                    elif results['WSREP_SST_DONOR_REJECTS_QUERIES'] == 'ON':
+                        raise base.Database.Error('WSREP_SST_DONOR_REJECTS_QUERIES')
+                if results['WSREP_REJECT_QUERIES'] != 'NONE':
+                    raise base.Database.Error('WSREP_REJECT_QUERIES: %s' % results['WSREP_REJECT_QUERIES'])
+                NODE_STATE.mark_online(node)
+                break
             except base.Database.Error as e:
                 LOGGER.info(e, exc_info=True)
                 NODE_STATE.mark_offline(node)
